#ifndef APF_H
#define APF_H

#include <vector>

class Grid;

struct GridPoint {
	Grid *grid;
	unsigned long x, y;
	inline GridPoint() {}
	inline GridPoint(const GridPoint& other) {
		grid = other.grid;
		x = other.x;
		y = other.y;
	}
	inline GridPoint(Grid* _grid, unsigned long _x, unsigned long _y) {
		grid = _grid;
		x = _x;
		y = _y;
	}
	inline bool operator==(const GridPoint& other) const {
		return (grid==other.grid && x==other.x && y==other.y);
	}
};

struct PassagePoint {
	GridPoint p;
	unsigned long cost;
};
class Passage {
	
	friend class Grid;
	
	private:
	Passage *prev, *next;
	unsigned long id;
	PassagePoint point1, point2;
	unsigned long dir;
	
	public:
	Passage(GridPoint p1, GridPoint p2, unsigned long cost1, unsigned long cost2, unsigned long _dir);
	~Passage();
	static Passage* Find(unsigned long id);
	
	public:
	inline unsigned long GetID() { return id; }
	inline Passage* GetPrevious() { return prev; }
	inline Passage* GetNext() { return next; }
	
};

class Grid {
	
	friend class Passage;
	
	public:
	struct Cell {
		unsigned short bottomborder, rightborder, bottomrightcorner;
		unsigned short temp;
		unsigned long cost;
		PassagePoint *passage[4];
		inline Cell() {
			bottomborder = 0;
			rightborder = 0;
			bottomrightcorner = 0;
			cost = 0;
			passage[0] = NULL;
			passage[1] = NULL;
			passage[2] = NULL;
			passage[3] = NULL;
		}
	};
	
	private:
	Grid *prev, *next;
	unsigned long id;
	unsigned long width, height;
	std::vector<Cell> data;
	
	public:
	Grid(unsigned long _width, unsigned long _height);
	~Grid();
	void Clear();
	static Grid* Find(unsigned long id);
	
	static bool FindPath(
		const std::vector<GridPoint>& startpoints,
		const std::vector<GridPoint>& endpoints,
		std::vector<unsigned char>* path,
		GridPoint* startpoint, GridPoint* endpoint, unsigned long* cost,
		unsigned long cost_normal, unsigned long cost_diagonal);
	
	public:
	inline unsigned long GetID() { return id; }
	inline Grid* GetPrevious() { return prev; }
	inline Grid* GetNext() { return next; }
	inline unsigned long GetWidth() { return width; }
	inline unsigned long GetHeight() { return height; }
	inline Cell& GetCell(unsigned long x, unsigned long y) { return data[x+y*width]; }
	
};

extern Grid *grids;
extern Passage *passages;

#endif // APF_H

